﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;
using Robomnia_1._0;
using Microsoft.WindowsMobile.SharedSource;

namespace Bluetooth
{
    class BluetoothFunctions
    {     

        static System.IO.Ports.SerialPort BTLink = new System.IO.Ports.SerialPort();
     
        public static bool ConnectQIK()
        {
            try
            {
                BTLink.BaudRate = 4800;
                BTLink.PortName = "COM6";
            }
            catch
            {
            }
            byte[] initialization = { 170 };
            if (!BTLink.IsOpen)
            {
                try
                {
                    BTLink.Open();
                    initialization[0] = 170;
                    BTLink.Write(initialization, 0, 1);
                    Sound.PlayVoice("Connected"); 
                    return true;
                }
                catch (Exception)
                {
                    return false;
                }
            }
            else 
            {
                return true;
            }

            
        }

        public static bool WriteServoPositionsandSpeeds(Hexapod.ServoArray goalpositions, Hexapod.ServoArray speeds)
        {
            try
            {   
                Writetoservospeeds(speeds); 
                Writetoheadservosspeeds(speeds.Head);
                Writetotailservosspeeds(speeds.Tail);
                
                Writetoservo(goalpositions);
                //Writetoservo(speeds);
                Writetoheadservos(goalpositions.Head);
                Writetotailservos(goalpositions.Tail);
                return true;
            }
            catch
            {
                return false;
            }
        }

        public static bool ConnectMaestro()
        {
            try
            {
                BTLink.BaudRate = 115200;
                BTLink.PortName = "COM6";
                BTLink.ReadTimeout = 100;
            }
            catch
            {
            }

            byte[] initialization = { 170 };
            if (!BTLink.IsOpen)
            {
                try
                {
                    BTLink.Open();
                    initialization[0] = 170;
                    BTLink.Write(initialization, 0, 1);
                    Sound.PlayVoice("Connected");
                    return true;
                }
                catch (Exception)
                {
                    return false;
                }
            }
            else
            {
                return true;
            }

        }

        public static void Close()
        {
            BTLink.Close();
        }

        public static int GetRange()//BTLink_DataReceived(object sender, System.IO.Ports.SerialDataReceivedEventArgs e)
        {
            BTLink.NewLine = "\rR";
            //Debug.WriteLine(BTLink.NewLine);
            string rangestring = "0";
            int range;
            if (BTLink.IsOpen)
            {
                try
                {
                    rangestring = BTLink.ReadLine();
                    //rangestring = rangestring.Remove(0, 1); 
                }
                catch (Exception ex)
                {
                    //Debug.WriteLine(ex.ToString());
                }
            }
            //Debug.WriteLine(rangestring);
            try
            {
                int factor = Convert.ToInt16(25.4);
                range = Convert.ToInt16(rangestring) * factor;           Debug.WriteLine("Range: " + range.ToString());
                
            }
            catch (Exception)
            {
                range = 255;
            }
            return range;
        }

        public static double GetMaestroInput(byte channelnumber) //read one of the Maestro's inputs.
        {
            byte[] Command = { 0x90, channelnumber }; //the command to read input
            double input;
            try
            {
                //BTLink.DiscardInBuffer(); //get rid of any crap in the buffer
                BTLink.Write(Command, 0, 2); //send the command
                byte[] Reply = { 0, 0 }; //make space for the reply
                BTLink.Read(Reply, 0, 2); //read the reply
                input = Reply[0] + (Reply[1] << 8); //convert the high and low bytes into a single number
                input /= 1024; //input has 1024 values so this corrects it to 0 - 1
                input *= 5; //input represents numbers from 0-5v so convert from 0-1 to 0-5
            }
            catch
            {
                input = 0;
            }
            return input; //return the value read
        }

        public static bool IsMaestroMoving() //read one of the Maestro's inputs.
        {
            //            Get Moving State
            //Compact protocol: 0x93 Pololu protocol: 0xAA, device number, 0x13 
            // Response: 0x00 if no servos are moving, 0x01 if servos are moving
            
            byte[] Reply = { 0 }; //make space for the reply
            byte[] Command = { 0x93 }; //the command to read input
            try
            {
                //BTLink.DiscardInBuffer(); //get rid of any crap in the buffer
                BTLink.Write(Command, 0, 1); //send the command    
                BTLink.Read(Reply, 0, 1); //read the reply
                
            }
            catch
            {

            }
            if (Reply[0] == 0x00) // 
            {
                return false;
            }
            else
            {
                return true;
            }
        }

        public static bool OpenBTLink()
        {
            try
            {
                BTLink.BaudRate = 9600;
                BTLink.PortName = "COM6";
            }
            catch
            {
            }

            byte[] initialization = { 170 };
            if (!BTLink.IsOpen)
            {
                try
                {
                    BTLink.Open();
                    Sound.PlayVoice("Connected");
                    return true;
                }
                catch (Exception)
                {
                    return false;
                }
            }
            else
            {
                return true;
            }
        }        

        public static byte[] ServoValuestoByteArray(Hexapod.LegServoValues ServoValues, int leg, byte Commandbyte)
        {

            byte[] Servonumbers = { 19, 16, 13, 10, 7, 4 }; // channel number of each legs coxa
            byte Coxanumber = Servonumbers[leg];
            byte Femurnumber = Convert.ToByte(Servonumbers[leg] - 1);
            byte Tibianumber = Convert.ToByte(Servonumbers[leg] - 2);

            int Coxalow = ServoValues.Coxa % 128; //convert to high/low bytes
            int Coxahigh = ServoValues.Coxa >> 7;
            int Femurlow = ServoValues.Femur % 128;
            int Femurhigh = ServoValues.Femur >> 7;
            int Tibialow = ServoValues.Tibia % 128;
            int Tibiahigh = ServoValues.Tibia >> 7;

            byte Coxalowbyte = 0; //initialise bytes
            byte Coxahighbyte = 0;
            byte Femurlowbyte = 0;
            byte Femurhighbyte = 0;
            byte Tibialowbyte = 0;
            byte Tibiahighbyte = 0;
            try //in case the servo pulse widths are ridiculous
            {
                Coxalowbyte = Convert.ToByte(Coxalow);
                Coxahighbyte = Convert.ToByte(Coxahigh);
                Femurlowbyte = Convert.ToByte(Femurlow);
                Femurhighbyte = Convert.ToByte(Femurhigh);
                Tibialowbyte = Convert.ToByte(Tibialow);
                Tibiahighbyte = Convert.ToByte(Tibiahigh);
            }
            catch (InvalidOperationException)
            {
                //Exception ex = new Exception('Servo out of range!');
                //throw ex;
            }

            byte[] bytearray = { Commandbyte, Coxanumber, Coxalowbyte, Coxahighbyte, Commandbyte, Femurnumber, Femurlowbyte, Femurhighbyte, Commandbyte, Tibianumber, Tibialowbyte, Tibiahighbyte };  //form the command string of bytes
            return bytearray;
        }

        #region Legs sent all at once - Positions
        public static void Writetoservo(Hexapod.ServoArray Servovalues) // Maestro
        {
            byte[] bytearray0 = { 0, 0, 0 };
            byte[] bytearray1 = { 0, 0, 0 };
            byte[] bytearray2 = { 0, 0, 0 };
            byte[] bytearray3 = { 0, 0, 0 };
            byte[] bytearray4 = { 0, 0, 0 };
            byte[] bytearray5 = { 0, 0, 0 };
            for (int leg = 0; leg<6; leg++)
            {
                switch (leg)
                {
                    case 0:
                        {
                            bytearray0 = ServoValuestoByteArray(Servovalues.Legs[0], leg, 132);
                        }
                        break;
                    case 1:
                        {
                            bytearray1 = ServoValuestoByteArray(Servovalues.Legs[1], leg, 132);
                        }
                        break;
                    case 2:
                        {
                            bytearray2 = ServoValuestoByteArray(Servovalues.Legs[2], leg, 132);
                        }
                        break;
                    case 3:
                        {
                            bytearray3 = ServoValuestoByteArray(Servovalues.Legs[3], leg, 132);
                        }
                        break;
                    case 4:
                        {
                            bytearray4 = ServoValuestoByteArray(Servovalues.Legs[4], leg, 132);
                        }
                        break;
                    case 5:
                        {
                            bytearray5 = ServoValuestoByteArray(Servovalues.Legs[5], leg, 132);
                        }
                        break;
                }
            }
            byte[] PositionByteArray = new byte[72];
            for (int i = 0; i <= 11; i++)
            {
                PositionByteArray[i] = bytearray0[i];
                PositionByteArray[i + (1 * 12)] = bytearray1[i];
                PositionByteArray[i + (2 * 12)] = bytearray2[i];
                PositionByteArray[i + (3 * 12)] = bytearray3[i];
                PositionByteArray[i + (4 * 12)] = bytearray4[i];
                PositionByteArray[i + (5 * 12)] = bytearray5[i];
            }

            BTLink.Write(PositionByteArray, 0, PositionByteArray.Length);
        }
        #endregion




        #region Legs sent all at once - Speeds
        public static void Writetoservospeeds(Hexapod.ServoArray Servospeeds) // Maestro
        {
            int leg;
            byte[] bytearray0 = { 0, 0, 0 };
            byte[] bytearray1 = { 0, 0, 0 };
            byte[] bytearray2 = { 0, 0, 0 };
            byte[] bytearray3 = { 0, 0, 0 };
            byte[] bytearray4 = { 0, 0, 0 };
            byte[] bytearray5 = { 0, 0, 0 };
            for (leg = 0; leg < 6; leg++)
            {
                switch (leg)
                {
                    case 0:
                        {
                            bytearray0 = ServoValuestoByteArray(Servospeeds.Legs[0], leg,135);
                        }
                        break;
                    case 1:
                        {
                            bytearray1 = ServoValuestoByteArray(Servospeeds.Legs[1], leg,135);
                        }
                        break;
                    case 2:
                        {
                            bytearray2 = ServoValuestoByteArray(Servospeeds.Legs[2], leg,135);
                        }
                        break;
                    case 3:
                        {
                            bytearray3 = ServoValuestoByteArray(Servospeeds.Legs[3], leg,135);
                        }
                        break;
                    case 4:
                        {
                            bytearray4 = ServoValuestoByteArray(Servospeeds.Legs[4], leg,135);
                        }
                        break;
                    case 5:
                        {
                            bytearray5 = ServoValuestoByteArray(Servospeeds.Legs[5], leg,135);
                        }
                        break;
                }
            }

            byte[] SpeedByteArray = new byte[72];
            for (int i = 0; i <= 11; i++)
            {
                SpeedByteArray[i] = bytearray0[i];
                SpeedByteArray[i + (1 * 12)] = bytearray1[i];
                SpeedByteArray[i + (2 * 12)] = bytearray2[i];
                SpeedByteArray[i + (3 * 12)] = bytearray3[i];
                SpeedByteArray[i + (4 * 12)] = bytearray4[i];
                SpeedByteArray[i + (5 * 12)] = bytearray5[i];
            }
            int n = (2 * 7);
            n = n + 6;
            BTLink.Write(SpeedByteArray, 0, SpeedByteArray.Length);
        }
        #endregion

        public static void Writetoheadservos(Hexapod.HeadorTailServoValues ServoValues) // Maestro
        {
            byte Pannumber = 21;
            byte Tiltnumber = 20;
            int Panlow = ServoValues.Pan % 128; //convert to high/low bytes
            int Panhigh = ServoValues.Pan >> 7;
            int Tiltlow = ServoValues.Tilt % 128;
            int Tilthigh = ServoValues.Tilt >> 7;

            byte Panlowbyte = 0; //initialise bytes
            byte Panhighbyte = 0;
            byte Tiltlowbyte = 0;
            byte Tilthighbyte = 0;
            try //in case the servo pulse widths are ridiculous
            {
                Panlowbyte = Convert.ToByte(Panlow);
                Panhighbyte = Convert.ToByte(Panhigh);
                Tiltlowbyte = Convert.ToByte(Tiltlow);
                Tilthighbyte = Convert.ToByte(Tilthigh);
            }
            catch (InvalidOperationException)
            {
                //Exception ex = new Exception('Servo out of range!');
                //throw ex;
            }

            byte[] bytearray = { 132, Pannumber, Panlowbyte, Panhighbyte, 132, Tiltnumber, Tiltlowbyte, Tilthighbyte};  //form the command string of bytes
            try
            {
                BTLink.Write(bytearray, 0, bytearray.Length);
                Debug.WriteLine("Bytearray: " + bytearray);
            }
            catch
            {
            }

        }

        public static void Writetotailservos(Hexapod.HeadorTailServoValues ServoValues) // Maestro
        {
            byte Pannumber = 22;
            byte Tiltnumber = 23;
            int Panlow = ServoValues.Pan % 128; //convert to high/low bytes
            int Panhigh = ServoValues.Pan >> 7;
            int Tiltlow = ServoValues.Tilt % 128;
            int Tilthigh = ServoValues.Tilt >> 7;

            byte Panlowbyte = 0; //initialise bytes
            byte Panhighbyte = 0;
            byte Tiltlowbyte = 0;
            byte Tilthighbyte = 0;
            try //in case the servo pulse widths are ridiculous
            {
                Panlowbyte = Convert.ToByte(Panlow);
                Panhighbyte = Convert.ToByte(Panhigh);
                Tiltlowbyte = Convert.ToByte(Tiltlow);
                Tilthighbyte = Convert.ToByte(Tilthigh);
            }
            catch (InvalidOperationException)
            {
                //Exception ex = new Exception('Servo out of range!');
                //throw ex;
            }

            byte[] bytearray = { 132, Pannumber, Panlowbyte, Panhighbyte, 132, Tiltnumber, Tiltlowbyte, Tilthighbyte };  //form the command string of bytes
            try
            {
                BTLink.Write(bytearray, 0, bytearray.Length);
                Debug.WriteLine("Bytearray: " + bytearray);
            }
            catch
            {
            }

        }

        public static void Writetoheadservosspeeds(Hexapod.HeadorTailServoValues ServoSpeeds) // Maestro
        {
            byte Pannumber = 21;
            byte Tiltnumber = 20;
            int Panlow = ServoSpeeds.Pan % 128; //convert to high/low bytes
            int Panhigh = ServoSpeeds.Pan >> 7;
            int Tiltlow = ServoSpeeds.Tilt % 128;
            int Tilthigh = ServoSpeeds.Tilt >> 7;

            byte Panlowbyte = 0; //initialise bytes
            byte Panhighbyte = 0;
            byte Tiltlowbyte = 0;
            byte Tilthighbyte = 0;
            try //in case the servo pulse widths are ridiculous
            {
                Panlowbyte = Convert.ToByte(Panlow);
                Panhighbyte = Convert.ToByte(Panhigh);
                Tiltlowbyte = Convert.ToByte(Tiltlow);
                Tilthighbyte = Convert.ToByte(Tilthigh);
            }
            catch (InvalidOperationException)
            {
                //Exception ex = new Exception('Servo out of range!');
                //throw ex;
            }

            byte[] bytearray = { 135, Pannumber, Panlowbyte, Panhighbyte, 135, Tiltnumber, Tiltlowbyte, Tilthighbyte };  //form the command string of bytes
            try
            {
                BTLink.Write(bytearray, 0, bytearray.Length);
                Debug.WriteLine("Bytearray: " + bytearray);
            }
            catch
            {
            }

        }

        public static void Writetotailservosspeeds(Hexapod.HeadorTailServoValues ServoSpeeds) // Maestro
        {
            byte Pannumber = 22;
            byte Tiltnumber = 23;
            int Panlow = ServoSpeeds.Pan % 128; //convert to high/low bytes
            int Panhigh = ServoSpeeds.Pan >> 7;
            int Tiltlow = ServoSpeeds.Tilt % 128;
            int Tilthigh = ServoSpeeds.Tilt >> 7;

            byte Panlowbyte = 0; //initialise bytes
            byte Panhighbyte = 0;
            byte Tiltlowbyte = 0;
            byte Tilthighbyte = 0;
            try //in case the servo pulse widths are ridiculous
            {
                Panlowbyte = Convert.ToByte(Panlow);
                Panhighbyte = Convert.ToByte(Panhigh);
                Tiltlowbyte = Convert.ToByte(Tiltlow);
                Tilthighbyte = Convert.ToByte(Tilthigh);
            }
            catch (InvalidOperationException)
            {
                //Exception ex = new Exception('Servo out of range!');
                //throw ex;
            }

            byte[] bytearray = { 135, Pannumber, Panlowbyte, Panhighbyte, 135, Tiltnumber, Tiltlowbyte, Tilthighbyte };  //form the command string of bytes
            try
            {
                BTLink.Write(bytearray, 0, bytearray.Length);
                Debug.WriteLine("Bytearray: " + bytearray);
            }
            catch
            {
            }

        }
        
    }
}

