﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.Text;
using System.Threading;

namespace Hexapod
{
    enum State
    {
        lookingleft,
        lookinglefttoturn,
        lookingright,
        roamingfree,
        lookingrighttocenter,
        lookinglefttocenter,
        turningleft,
        turningright,
        headcentered
    }

    enum GaitType
    {
        None,
        Tripod,
        TwoLegRipple,
        OneLegRipple,
        FourLeg
    }
     

    class Body
    {
        

        #region Variables
        public State currentstate = State.roamingfree;
        public GaitType gait; //= GaitType.None;
        public IntVector BodyShift = new IntVector(0, 0, 0);             //intialise with no shift
        public IntVector BodyRotation = new IntVector(0, 0, 0);          // no rotation (level)
        protected IntVector Legrotations = new IntVector(0, 0, 0);
        protected IntVector Legshift = new IntVector(0, 0, 0);

        public int speed = 0;                       // speed of zero (no walking - stepping is suppressed elsewhere
        public int speedtarget;                     // setting this changes the speed smoothly according to speedacceleration
        public int speedacceleration;

        public int gaitdirection = 0;               // walking direction is forward
        public int stepheight = 20;                 // stepheight is 20mm
        public int steplength = 0;

        public int gaitrotation = 0;                // turning speed is zero
        public int gaitrotationtarget;
        public int gaitrotationacceleration;

        public int stepsincycle = 6;                // stepsincycle 
        protected int t = 0;                        // intitial time is zero
        int[] ZeroIntArray = {0,0,0};       //
        private double[] ZeroDoubleArray = {0,0,0}; //
        private ServoArray oldpositions = new ServoArray(6000);
        private ServoArray newpositions = new ServoArray(0);    //
        private ServoArray servospeeds = new ServoArray(10);
        public bool updated;

        public int rangeleft;
        public int rangeright;
        
        public bool headcentered;
        public int u = 0; // a counter which determines how long the robot is to look after itself like when turning without the nav program updating it
        public Leg[] legs;
        public HeadorTail head;
        public HeadorTail tail;
        #endregion

        
        public Body()
        {
            // make some legs!
            legs = new Leg[6];
            for (int i = 0; i < 6; i++)
            {
                legs[i] = new Leg(i);
            }
            head = new HeadorTail("Head", 30, 50);
            tail = new HeadorTail("Tail", 20, 10);

            //make some gaits!

        }
        
        //public Body(int NoOfLegs, IntVector[] LegOrientations, IntVector[] CoxaPositions)
        //{
        //    legs = new Leg[6];
        //    for (int i = 0; i < 6; i++)
        //    {
        //        legs[i] = new Leg(i);
        //    }
        //}


        public void Update(int calculationinterval) //method to update the servo positions and speeds
        {
            Accelation();
            stepsincycle = Gait.Gaittostepsincycle(gait);
            steplength = Speedtosteplength(speed);
            
            Legrotations.X = - BodyRotation.X;
            Legrotations.Y = - BodyRotation.Y;
            Legrotations.Z = -BodyRotation.Z * 1 / 3; //body does 1/3 of the overall rotation

            Legshift.X = -BodyShift.X;
            Legshift.Y = -BodyShift.Y;
            Legshift.Z = -BodyShift.Z;

            head.Angles.Pan = BodyRotation.Z * 2 / 3;
            head.Angles.Tilt = BodyRotation.X;

            tail.Angles.Pan = BodyRotation.Z * 2 / 3;
            tail.Angles.Tilt = BodyRotation.X;


            if (speed == 0 && gaitrotation == 0)
            {
                stepheight = 0;
            }

            if (gait == GaitType.FourLeg)
            {
                legs[0].Coordinates.Z = -120;
                legs[0].Coordinates.Y = legs[0].HomePosition.Y;
                legs[0].Coordinates.X = legs[0].Offset.X;
                legs[0].walking = false;
                legs[1].Coordinates.Z = 0;
                legs[1].Coordinates.Y = legs[1].HomePosition.Y + 50;
                legs[1].Coordinates.X = legs[1].Offset.X;
                legs[1].walking = false;
                legs[2].Coordinates.Y = legs[2].HomePosition.Y + 70;
                legs[2].Coordinates.X = legs[2].HomePosition.X - 20;
                legs[3].Coordinates.Y = legs[3].HomePosition.Y + 70;
                legs[3].Coordinates.X = legs[3].HomePosition.X + 20;

                steplength = 30;
                stepheight = 40;
                int Shiftx = 20;
                int Shifty = 10;
                double[] XShift = { 1, 1, 1, 1, 1, 1, 1, 1, 0.33, -0.33,  -1,  -1, -1, -1, -1, -1, -1, -1, -0.33, 0.33 }; 
                double[] YShift = { -1, -1, 0, 1, 1, 0, -1, -1, 0,  1,  1, 0 };
                IntVector GaitShift = new IntVector(Convert.ToInt16( XShift[t] * Shiftx ), 0, 0);//YShift[t]*Shifty,0);
                IntVector TotalShift = Legshift.Add(GaitShift);
                
                for (int i = 0; i < 2; i++)
                {
                    legs[i].Update(stepsincycle,gait,stepheight,steplength,gaitdirection,gaitrotation,Legshift,Legrotations,t);
                }

                for (int i = 2; i < 6; i++)
                {
                    legs[i].Update(stepsincycle, gait, stepheight, steplength, gaitdirection, gaitrotation, TotalShift, Legrotations, t);
                }
            }

            else
            {
                for (int i = 0; i < 6; i++)
                {
                    legs[i].Update(stepsincycle, gait, stepheight, steplength, gaitdirection, gaitrotation, Legshift, Legrotations, t);
                }
            }

            for (int i = 0; i<6; i++)
            {
                newpositions.Legs[i] = legs[i].ServoValues;
            }

            newpositions.Head = head.Values;
            newpositions.Tail = tail.Values;
            servospeeds = CalculateServoSpeeds(oldpositions, newpositions,calculationinterval);

            Bluetooth.BluetoothFunctions.WriteServoPositionsandSpeeds(newpositions, servospeeds);

            oldpositions = newpositions.Clone();

            t++;
            t = t % stepsincycle;
        }

 
            

        private void Accelation() //method to smoothly change the speed and rotation rate
        {
            if (speed < speedtarget)
            {
                speed = speed + speedacceleration;
                if (speed > speedtarget)
                {
                    speed = speedtarget;
                }
            }
            else if (speed > speedtarget)
            {
                speed = speed - speedacceleration;
                if (speed < speedtarget)
                {
                    speed = speedtarget;
                }
            }

            if (gaitrotation < gaitrotationtarget)
            {
                gaitrotation = gaitrotation + gaitrotationacceleration;
                if (gaitrotation > gaitrotationtarget)
                {
                    gaitrotation = gaitrotationtarget;
                }
            }
            else if (gaitrotation > gaitrotationtarget)
            {
                gaitrotation = gaitrotation - gaitrotationacceleration;
                if (gaitrotation < gaitrotationtarget)
                {
                    gaitrotation = gaitrotationtarget;
                }
            }
        }

        private static int Speedtosteplength(int speed) //speed as a percentage
        {
            int maxsteplength = 45;
            int steplength = maxsteplength * speed / 100;
            return steplength;
        }

        public static ServoArray CalculateServoSpeeds(ServoArray oldpositions, ServoArray newpositions, int time)
        {
            time /= 10; //Maestro takes time in terms of 10ms

            if (time == 0)  //what to do if we want the move done straight away.
            {
                ServoArray zeros = new ServoArray();
                return zeros; ; //just return zeros. The Maestro interprets zeros as maximum speed.
            }
            else   //if we have an actual time to take
            {
                ServoArray calculatedspeeds = new ServoArray(); //Create an array. The constructor fills it with zeros.
                for (int i = 0; i < 6; i++)   //cycle through the 3 ints in each leg represent the speeds of the coxa, femur, tibia
                {
                    calculatedspeeds.Legs[i].Coxa = (Math.Abs(newpositions.Legs[i].Coxa - oldpositions.Legs[i].Coxa)) / time; //set each element to the difference in positions divided by the time.
                    calculatedspeeds.Legs[i].Femur = (Math.Abs(newpositions.Legs[i].Femur - oldpositions.Legs[i].Femur)) / time;
                    calculatedspeeds.Legs[i].Tibia = (Math.Abs(newpositions.Legs[i].Tibia - oldpositions.Legs[i].Tibia)) / time;
                }
                
                calculatedspeeds.Head.Pan = (Math.Abs(newpositions.Head.Pan - oldpositions.Head.Pan)) / time;
                calculatedspeeds.Tail.Tilt = (Math.Abs(newpositions.Tail.Tilt - oldpositions.Tail.Tilt)) / time;
                calculatedspeeds.Head.Aux = (Math.Abs(newpositions.Head.Aux - oldpositions.Head.Aux)) / time;
                calculatedspeeds.Tail.Pan = (Math.Abs(newpositions.Tail.Pan - oldpositions.Tail.Pan)) / time;
                calculatedspeeds.Head.Tilt = (Math.Abs(newpositions.Head.Tilt - oldpositions.Head.Tilt)) / time;
                calculatedspeeds.Tail.Aux = (Math.Abs(newpositions.Tail.Aux - oldpositions.Tail.Aux)) / time;
                
                return calculatedspeeds;
            }
        }

        //public static ServoArray FullServoArray(int value)
        //{
        //    ServoArray zeros = new ServoArray();
        //    int i;
        //    for (i = 0; i < 6; i++)
        //    {
        //        LegServoValues values = new LegServoValues( value, value, value );
        //        zeros.Legs[i] = values;
        //    }
        //    zeros.Head.Pan = value; zeros.Head.Tilt = value; zeros.Head.Aux = value;
        //    zeros.Tail.Pan = value; zeros.Tail.Tilt = value; zeros.Tail.Aux = value;
               
        //    return zeros;
        //}

    }
}
