﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.Text;

namespace Hexapod
{
    class Gait
    {
        double[] GaitH;
        double[] GaitV;
        int Stepsincycle;

        public Gait(double[] gaith, double[] gaitv, int stepsincycle)
        {
            GaitH = gaith;
            GaitV = gaitv;
            Stepsincycle = stepsincycle;
        }








        public static DoubleVector EndGait(int localt, GaitType gait, int direction, int stepsincycle, int steplength, int stepheight, int leg)
        //takes the gait, direction 0-360, speed 0 -100, steplength (mm),stepheight (mm), leg
        {
            DoubleVector gaitVH = GaitVH(localt, leg, stepsincycle, stepheight, steplength, gait);
            DoubleVector gaitXYZ = GaitXYZ(gaitVH, direction);
            return gaitXYZ;
        }

        public static DoubleVector GaitrotationZ(int localt, int stepsincycle, GaitType gait, int degreesperstep, int leg, DoubleVector Coords)
        //Takes the (gaited) coordinates and rotates them by a certain amount, so that the rotation reaches degreesperstep as the leg is put down and the gradually returns to zero
        {
            int theta = GetGaitTheta(gait);
            int transitiontime = (theta * stepsincycle) / 360;
            int zrotation;
            double m1 = CalculateGradient(0, 0, transitiontime, degreesperstep); //calculate the rate of turn for 0 --> transitiontime
            double m2 = CalculateGradient(transitiontime, degreesperstep, stepsincycle, 0); //calculate the rate of turn for transitiontime --> stepincycle(T)
            if (0 <= localt && localt <= transitiontime) //where in the cycle are we?
            {
                zrotation = Convert.ToInt32(m1 * localt);  //y-b = m(x-a)  // this is when the leg is raised.
            }
            else
            {
                zrotation = Convert.ToInt32(m2 * (localt - transitiontime) + degreesperstep); //y-b = m(x-a) //this is where the leg is lowered
            }
            DoubleVector RotatedCoords = Zrotate(Coords, zrotation);
            return RotatedCoords;
        }


        #region New gait
        private static double CalculateGaitV(int localt, int leg, double stepheight, int stepsincycle, GaitType gait)
        //Calculated the vertical part of the legs gait offset
        {
            switch (gait)
            {
                case (GaitType.Tripod):
                    {
                        double[] Gaitv = {0, 0.8, 1, 0, 0, 0};
                        int gaitv = Convert.ToInt16(Gaitv[localt] * stepheight);
                        return -gaitv;
                    }
                    break;
                case (GaitType.TwoLegRipple):
                    {
                        double[] Gaitv = { 0, 0.4, 1, 0, 0, 0, 0, 0, 0 };
                        int gaitv = Convert.ToInt16(Gaitv[localt] * stepheight);
                        return -gaitv;
                    }
                    break;
                case (GaitType.FourLeg):
                    {
                        double[] Gaitv = { 0, 0.4, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
                        int gaitv = Convert.ToInt16(Gaitv[localt] * stepheight);
                        return -gaitv;
                    }
                    break;
                default:
                    {
                        int gaitv = 0;
                        return gaitv;
                    }
                    break;
                
            }
        }

        private static double CalculateGaitH(int localt, int leg, double steplength, int stepsincycle, GaitType gait)
        //Calculates the horizontal part of the gait offset
        {
            switch (gait)
            {
                case (GaitType.Tripod):
                    {
                        double[] Gaith = { -0.5, -0.1, 0.4, 0.5, 0.33, -0.33 };
                        int gaith = Convert.ToInt16(Gaith[localt] * steplength);
                        return gaith;
                    }
                    break;
                case (GaitType.TwoLegRipple):
                    {
                        double[] Gaith = { -0.5, -0.1, 0.4, 0.5, 0.33, 0.167, 0, -0.167, -0.33};
                        int gaith = Convert.ToInt16(Gaith[localt] * steplength);
                        return gaith;
                    }
                    break;
                case (GaitType.FourLeg):
                    {
                        double m = CalculateGradient(1, 0.5, 20 - 3, -0.5);
                        double[] Gaith = { -0.5, -0.1, 0.4, 0.5, m * 1 + 0.5, m * 2 + 0.5, m * 3 + 0.5, m * 4 + 0.5, m * 5 + 0.5, m * 6 + 0.5, m * 7 + 0.5, m * 8 + 0.5, m * 9 + 0.5, m * 10 + 0.5, m * 11 + 0.5, m * 12 + 0.5, m * 13 + 0.5, m * 14 + 0.5, m * 15 + 0.5, m * 16 + 0.5};//, m * 17 + 0.5, m * 18 + 0.5, m * 19 + 0.5 };
                        //double[] Gaith = { -0.5, -0.1, 0.4, 0.5, 0.389, 0.278, 0.167, 0.056, -0.056, -0.167, -0.278, -0.389 };
                        int gaith = Convert.ToInt16(Gaith[localt] * steplength);
                        return gaith;
                    }
                    break;
                default:
                    {
                        int gaith = 0;
                        return gaith;
                    }
                    break;

            }
        }
        #endregion
        

        private static double CalculateGradient(double x1,double y1, double x2, double y2)
        //Used where there is a linear movement
        {
	        double m = (y2 - y1) / (x2 - x1);
	        return m;
        }

        public static int GetGaitOffset(GaitType gait , int leg)
        //Gets the legs position in the gait pattern
        {
            int gaitoffset;
	        switch (gait)
	        {
                case GaitType.Tripod:
                    int[] GaitOffsets0 = { 0,  180, 180, 0, 0, 180 }; //in degrees
                    gaitoffset = GaitOffsets0[leg];
                    return gaitoffset;
                case GaitType.TwoLegRipple:
                    int[] GaitOffsets1 = { 120, 300, 240, 60, 0, 180 };
                    gaitoffset = GaitOffsets1[leg];
                    return gaitoffset;
                case GaitType.OneLegRipple:
                    int[] GaitOffsets2 = { 120, 300, 240, 60, 0, 180 }; 
                    gaitoffset = GaitOffsets2[leg];
                    return gaitoffset;
                case GaitType.FourLeg:
                    int[] GaitOffsets3 = { 0, 0, 90, 270, 0, 180 };
                    gaitoffset = GaitOffsets3[leg];
                    return gaitoffset;
		        default:
                    gaitoffset = 0;	
                    return gaitoffset;
		        //break;
	        }
        }

        public static int GetGaitTheta(GaitType gait )
        //Gets the degrees in the cycle for which the leg is in transition
        {
            int gaittheta = 0;
	        switch (gait)
	        {
		        case GaitType.Tripod:
                    gaittheta = 180; //in degrees
                    return gaittheta;
		        //break;
                case GaitType.TwoLegRipple:
                    gaittheta = 120;
                    return gaittheta;
                case GaitType.OneLegRipple:
                    gaittheta = 60;
                    return gaittheta;
                case GaitType.FourLeg:
                    gaittheta = 90;
                    return gaittheta;
		        default:
			        gaittheta = 60;
                    return gaittheta;
		        //break;
	        }
        }

        public static int Gaittostepsincycle(Hexapod.GaitType gait)
        {
            int minstepsincycle = 6;
            int stepsincycle = 6;
            switch (gait)
            {
                case GaitType.Tripod:
                    stepsincycle = minstepsincycle;
                    break;
                case GaitType.TwoLegRipple:
                    stepsincycle = minstepsincycle * 3 / 2;
                    break;
                case GaitType.OneLegRipple:
                    stepsincycle = minstepsincycle * 6 / 2;
                    break;
                case GaitType.FourLeg:
                    //stepsincycle = minstepsincycle * 4 / 2;
                    stepsincycle = 20;
                    break;
            }
            return stepsincycle;
        }

        private static DoubleVector GaitVH(int t, int leg, int stepsincycle, double stepheight,  double steplength, GaitType gait) 
        //combine the horizontal and vertical displacements into a struct
        {
            double gaitv = CalculateGaitV(t, leg, stepheight, stepsincycle, gait);
            double gaith = CalculateGaitH(t, leg, steplength, stepsincycle, gait); 
	        DoubleVector Gaitvh = new DoubleVector(gaitv, gaith,0);
            return Gaitvh;
        }


        private static DoubleVector GaitXYZ(DoubleVector GaitVH, int direction)
        //converts the horizontal and vertical components of the gait offset to xyz based on direction
        {
            double directionradians = direction * 2 * Math.PI / 360;
            double xdisplacement = Math.Sin(directionradians) * GaitVH.Y;
            double ydisplacement = Math.Cos(directionradians) * GaitVH.Y;
            double zdisplacement = GaitVH.X;
            DoubleVector XYZdisplacement = new DoubleVector(xdisplacement, ydisplacement, zdisplacement);
            return XYZdisplacement;
        }

        
        private static DoubleVector Zrotate(DoubleVector Coords, int zrotation)
        //performs the actual rotation. Borrowed from BodyRotations.cs
        {
            if (zrotation == 0)
            {
                return Coords;
            }
            else
            {
                double x = Coords.X;
                double y = Coords.Y;
                double z = Coords.Z;
                double rotationradians = zrotation * 2 * Math.PI / 360;
                double newx = x * Math.Cos(rotationradians) - y * Math.Sin(rotationradians);
                double newy = y * Math.Cos(rotationradians) + x * Math.Sin(rotationradians);
                double newz = z;
                DoubleVector NewCoordinates = new DoubleVector(newx, newy, newz);
                return NewCoordinates;
            }
        }
    }
}

