﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;

namespace Hexapod
{
    
    class IK
    {
        static int CoxaFemurOffset = 38;
        static int l1 = 70;
        static int l2 = 90;

        public static LegServoAngles InverseKinematics(DoubleVector Coordinates)  //returns servo angles in degrees given coordinates
        {
            double x = Coordinates.X;
            double y = Coordinates.Y;
            double z = Coordinates.Z;
            double Coxavalue = Coxa(x,y);
            double Femurvalue = Femur(x, y, z);
            double Tibiavalue = Tibia(x, y, z);
            LegServoAngles servoAngles = new LegServoAngles(Coxavalue, Femurvalue, Tibiavalue);
            return servoAngles;
        }

        private static double Coxa(double x, double y)
        {
            double coxarad = Math.Atan2(y,x);
            double coxa = ((coxarad * 360) / (2 * Math.PI));
            return coxa;
        }      

        private static double Femur(double x, double y, double z)
        {
            double r3d = Length3d(x,y,z);
            double femurrad = Math.Acos((Math.Pow(l1, 2) + Math.Pow(r3d, 2) - Math.Pow(l2, 2)) / (2 * r3d * l1)) +Math.Atan2(Length2d(x, y), z);
            double femur = ((femurrad * 360) / (2 * Math.PI));
            return femur;
        }

        private static double Tibia(double x, double y, double z)
        {
            double tibiarad = Math.Acos((Math.Pow(l1, 2) + Math.Pow(l2, 2) - Math.Pow(Length3d(x,y,z), 2)) / (2 * l2 * l1));
            double tibia = ((tibiarad * 360) / (2 * Math.PI));
            return tibia;
        }
        
        private static double Length3d(double x, double y, double z)
        {
            double r = Math.Sqrt(Math.Pow(Length2d(x,y),2) + Math.Pow(z, 2));
            return r;
        }

        private static double Length2d(double x, double y)
        {
            double r = (Math.Sqrt(Math.Pow(x, 2) + Math.Pow(y, 2)))-CoxaFemurOffset;
            return r;
        }
    }
}
