﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.Text;

namespace Hexapod
{
    class Leg
    {
        #region Variables
        int LegNumber;
        public IntVector Coordinates;
        DoubleVector GaitedCoordinates;
        DoubleVector RotatedCoordinates;
        public IntVector HomePosition;
        public LegServoValues ServoValues;
        IntVector LegOrientation;
        public IntVector Offset;
        Hexapod.GaitType Gait;
        int GaitDirection;
        int GaitRotation;
        int offset;
        int localt;
        public bool walking = true;
        IntVector Shift;
        IntVector Rotation;
        public bool raised;
        #endregion

        public Leg(int legNumber)
        {
            LegNumber = legNumber;
            Offset = LegParameters.GetLegOffset(LegNumber);
            LegOrientation = Hexapod.LegParameters.GetLegOrientation(LegNumber);
            HomePosition = new IntVector(0,0,0);
            HomePosition = Hexapod.LegParameters.GetLegHomePosition(LegNumber);
            Coordinates = new IntVector(0,0,0);

            Coordinates = Hexapod.LegParameters.GetLegHomePosition(LegNumber);
            //Coordinates = HomePosition;
        }

        public void MoveHome()
        {
            //Coordinates = HomePosition;
            Coordinates = Hexapod.LegParameters.GetLegHomePosition(LegNumber);
        }


        public void Update(int stepsincycle, GaitType gait, int stepheight, int steplength, int direction, int gaitrotation, IntVector shift, IntVector rotation, int t)
        {
            //Coordinates = HomePosition;
            offset = Hexapod.Gait.GetGaitOffset(gait, LegNumber);
            Gait = gait;
            GaitDirection = direction;
            GaitRotation = gaitrotation;
            
            localt = ((stepsincycle + t) - (offset*stepsincycle / 360)) % stepsincycle;
            if (walking) //only work out all the gait stuff if we have to
            {
                DoubleVector Gaitfactor = Hexapod.Gait.EndGait(localt, gait, direction, stepsincycle, steplength, stepheight, LegNumber);
                DoubleVector PositionGaited = Coordinates.Add(Gaitfactor);
                DoubleVector Gaitedrotated = Hexapod.Gait.GaitrotationZ(localt, stepsincycle, gait, gaitrotation, LegNumber, PositionGaited);
                GaitedCoordinates = Gaitedrotated;
            }
            else
            {
                GaitedCoordinates = Coordinates.ToDoubleVector();
            }

            if (GaitedCoordinates.Z == HomePosition.Z)//== Coordinates.Z)
            {
                raised = false; //if the foot's down, it aint up.
            }
            else
            {
                raised = true;
            }
        
            Shift = shift;
            Rotation = rotation;
            RotatedCoordinates = GaitedCoordinates.Rotate(Rotation);
            DoubleVector PositionShifted = RotatedCoordinates.Add(Shift);
            DoubleVector LocalPosition = PositionShifted.Add(-1 * Offset.X, -1 * Offset.Y, -1 * Offset.Z);
            DoubleVector FinalLocalPosition = CorrectforOrientation(LocalPosition, LegOrientation);
            LegServoAngles ServoAngles = Hexapod.IK.InverseKinematics(FinalLocalPosition);
            ServoValues = ServoAngles.ConvertToLegServoValues(LegNumber);

        }

        public static DoubleVector CorrectforOrientation(DoubleVector Coordinates, IntVector LegOrientation)
        {    
            double localorienx = Coordinates.X * LegOrientation.X;
            double localorieny = Coordinates.Y * LegOrientation.Y;
            double localorienz = Coordinates.Z * LegOrientation.Z;
            DoubleVector LocalCoordinates = new DoubleVector(localorienx, localorieny, localorienz);
            return LocalCoordinates;
        }
    }
}
