﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.Text;

namespace Hexapod
{
    class LegParameters
    {
        public static IntVector GetLegOffset(int leg)
        {
            //input the xyz positions of the coxa servos on the hexapod
            int[] Xoffsets = { 40, -40, 40, -40, 40, -40 };
            int[] Yoffsets = { 50, 50, 0, 0, -50, -50 };
            int[] Zoffsets = { 0, 0, 0, 0, 0, 0 };
            IntVector LegOffset = new IntVector(Xoffsets[leg], Yoffsets[leg], Zoffsets[leg]);
            return LegOffset;
        }

        public static IntVector GetLegOrientation(int leg)
        {
            int[] Xorientation = { 1, -1, 1, -1, 1, -1 }; //Set the leg directions here eg. those on left are non-inverted, those on right are inverted in y and x directions
            int[] Yorientation = { 1, 1, 1, 1, 1, 1 };
            int[] Zorientation = { 1, 1, 1, 1, 1, 1 };
            IntVector LegOrientation = new IntVector( Xorientation[leg], Yorientation[leg], Zorientation[leg] );
            return LegOrientation;
        }

        public static LegServoValues GetServoValueAtLimbCentre(int leg) //Set the servo inputs placing limbs at reference position
        {
            int[] CoxaValues = { 2014, 1144, 1644, 1570, 1241, 1809 };
            int[] FemurValues = { 1980, 1098, 1950, 1200, 1991, 1220  };
            int[] TibiaValues = { 1285, 1776, 1116, 1731, 1383, 1677 };
            LegServoValues  Values = new LegServoValues(CoxaValues[leg], FemurValues[leg], TibiaValues[leg]);
            return Values;
        }

        public static HeadorTailServoValues GetHeadServoValuesAtCentre()
        {
            int Pan = 1430;
            int Tilt = 1570;
            int Aux = 0;
            HeadorTailServoValues Values = new HeadorTailServoValues( Pan, Tilt, 0, "" );
            return Values;
        }

        public static HeadorTailServoValues GetTailServoValuesAtCentre() 
        {
            int Pan = 1500;
            int Tilt = 1500;
            int Aux = 0;
            HeadorTailServoValues Values = new HeadorTailServoValues( Pan, Tilt, 0, "" );
            return Values;
        }

        public static IntVector GetServoDirections(int leg)
        {
            int[] CoxaDirections = { -1, 1, -1, 1, -1, 1};
            int[] FemurDirections = {1, -1, 1, -1, 1, -1};
            int[] TibiaDirections = { -1, 1, -1, 1, -1, 1 };
            IntVector Directions = new IntVector( CoxaDirections[leg], FemurDirections[leg], TibiaDirections[leg] );
            return Directions;
        }

        public static IntVector GetLegHomePosition(int leg)
        {
            int[] Xpositions = { 100, -100, 110, -110, 95, -95 };
            int[] Ypositions = { 90, 90, 0, 0, -110, -110 };
            int[] Zpositions = { 40, 40, 40, 40, 40, 40 };
            IntVector LegHomePosition = new IntVector(Xpositions[leg], Ypositions[leg], Zpositions[leg]);
            return LegHomePosition;
        }
    }
}
