﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.Text;

namespace Hexapod
{
    class ServoArray
    {
        public LegServoValues[] Legs;
        public HeadorTailServoValues Head;
        public HeadorTailServoValues Tail;

        public ServoArray()
        {
            Legs = new LegServoValues[6];
            for (int i = 0; i < 6; i++)
            {
                Legs[i] = new LegServoValues(0, 0, 0);
            }
            Head = new HeadorTailServoValues(0, 0, 0, "Head");
            Tail = new HeadorTailServoValues(0, 0, 0, "Tail");
        }

        public ServoArray(int value)
        {
            LegServoValues[] legs = new LegServoValues[6];
            for (int i = 0; i < 6; i++)
            {
                legs[i] = new LegServoValues(value, value, value);
            }
            Legs = legs;
            Head = new HeadorTailServoValues(value, value, value, "Head");
            Tail = new HeadorTailServoValues(value, value, value, "Tail");
        }
        public ServoArray Clone()
        {
            ServoArray ans = new ServoArray();
            for (int i = 0; i < 6; i++)
            {
                ans.Legs[i].Coxa = Legs[i].Coxa;
                ans.Legs[i].Femur = Legs[i].Femur;
                ans.Legs[i].Tibia = Legs[i].Tibia;
            }
            ans.Head.Pan = Head.Pan;
            ans.Head.Tilt = Head.Tilt;
            ans.Head.Aux = Head.Aux;
            ans.Tail.Pan= Tail.Pan;
            ans.Tail.Tilt= Tail.Tilt;
            ans.Tail.Aux = Tail.Aux;
            return ans;
        }
    }

    class DoubleVector
    {
        public double X;
        public double Y;
        public double Z;

        public DoubleVector(double x, double y, double z)
        {
            X = x;
            Y = y;
            Z = z;
        }

        public DoubleVector Add(DoubleVector toAdd)
        {
            DoubleVector addition = new DoubleVector(0,0,0);
            addition.X = X + toAdd.X;
            addition.Y = Y + toAdd.Y;
            addition.Z = Z + toAdd.Z;
            return addition;
        }

        public DoubleVector Add(IntVector toAdd)
        {
            DoubleVector addition = new DoubleVector(0,0,0);
            addition.X = X + toAdd.X;
            addition.Y = Y + toAdd.Y;
            addition.Z = Z + toAdd.Z;
            return addition;
        }

        public DoubleVector Add(int x, int y, int z)
        {
            DoubleVector addition = new DoubleVector(0,0,0);
            addition.X = X + x;
            addition.Y = Y + y;
            addition.Z = Z + z;
            return addition;
        }

        public DoubleVector Add(double x, double y, double z)
        {
            DoubleVector addition = new DoubleVector(0,0,0);
            addition.X = X + x;
            addition.Y = Y + y;
            addition.Z = Z + z;
            return addition;
        }

        public DoubleVector Rotate(IntVector Rotation)
        {
            DoubleVector RotRadians = new DoubleVector(0, 0, 0);
            DoubleVector Ans = new DoubleVector(0, 0, 0);

            RotRadians.X = Rotation.X * 2 * Math.PI / 360;
            RotRadians.Y = Rotation.Y * 2 * Math.PI / 360;
            RotRadians.Z = Rotation.Z * 2 * Math.PI / 360;

            double cX = Math.Cos(RotRadians.X);
            double sX = Math.Sin(RotRadians.X);

            double cY = Math.Cos(RotRadians.Y);
            double sY = Math.Sin(RotRadians.Y);

            double cZ = Math.Cos(RotRadians.Z);
            double sZ = Math.Sin(RotRadians.Z);

            Ans.X =  cZ * cY * X + (cZ * sY * sX - sZ * cX) * Y + (cZ * sY * cX + sZ * sX) * Z;
            Ans.Y =  sZ * cY * X + (sZ * sY * sX + cZ * cX) * Y + (sZ * sY * cX - cZ * sX) * Z;
            Ans.Z = -sY *     X + cY * sX *                  Y + cY * cX *                  Z;

            return Ans;
        }

        public IntVector ToIntVector()
        {
            int x = Convert.ToInt16(X);
            int y = Convert.ToInt16(Y);
            int z = Convert.ToInt16(Z);
            IntVector intVector = new IntVector(x, y, z);
            return intVector;
        }

        public double[] ToDouble()
        {
            double[] Double = { X, Y, Z };
            return Double;
        }

        public int[] ToInt()
        {
            int[] Int = { Convert.ToInt16(X), Convert.ToInt16(Y), Convert.ToInt16(Z) };
            return Int;
        }

    }

    class IntVector
    {
        public int X;
        public int Y;
        public int Z;

        public IntVector(int x, int y, int z)
        {
            X = x;
            Y = y;
            Z = z;
        }

        public DoubleVector Add(DoubleVector toAdd)
        {
            DoubleVector addition = new DoubleVector(0,0,0);
            addition.X = X + toAdd.X;
            addition.Y = Y + toAdd.Y;
            addition.Z = Z + toAdd.Z;
            return addition;
        }

        public IntVector Add(IntVector toAdd)
        {
            IntVector addition = new IntVector(0,0,0);
            addition.X = X + toAdd.X;
            addition.Y = Y + toAdd.Y;
            addition.Z = Z + toAdd.Z;
            return addition;
        }

        public IntVector Add(int x, int y, int z)
        {
            IntVector addition = new IntVector(0,0,0);
            addition.X = X + x;
            addition.Y = Y + y;
            addition.Z = Z + z;
            return addition;
        }

        public DoubleVector Add(double x, double y, double z)
        {
            DoubleVector addition = new DoubleVector(0,0,0);
            addition.X = X + x;
            addition.Y = Y + y;
            addition.Z = Z + z;
            return addition;
        }

        public DoubleVector ToDoubleVector()
        {
            double x = Convert.ToDouble(X);
            double y = Convert.ToDouble(Y);
            double z = Convert.ToDouble(Z);
            DoubleVector doubleVector = new DoubleVector(x, y, z);
            return doubleVector;
        }

        public int[] ToInt()
        {
            int[] Int = { X, Y, Z };
            return Int;
        }

        public double[] ToDouble()
        {
            double[] Double = { Convert.ToDouble(X), Convert.ToDouble(Y), Convert.ToDouble(Z) };
            return Double;
        }


    }

    class LegServoAngles
    {
        public double Coxa;
        public double Femur;
        public double Tibia;

        public LegServoAngles(double coxa, double femur, double tibia)
        {
            Coxa = coxa;
            Femur = femur;
            Tibia = tibia;
        }

        public LegServoValues ConvertToLegServoValues(int LegNumber)
        {
            LegServoValues ZeroAngles = Hexapod.LegParameters.GetServoValueAtLimbCentre(LegNumber);
            IntVector Direction = Hexapod.LegParameters.GetServoDirections(LegNumber); //servo reversing
            LegServoAngles workingangles = new LegServoAngles(0, 0, 0);
            workingangles.Femur = Femur - 180; //making up for difference between the zero angles used for calibrating and the zero angles as seen by the IK
            workingangles.Tibia = Tibia - 90;
            workingangles.Coxa = Coxa;

            ZeroAngles.Coxa *= 4; //from us to 1/4 us
            ZeroAngles.Femur *= 4;
            ZeroAngles.Tibia *= 4;

            double ScalingFactor = (50.49); //Maestro
            int CoxaServo = Convert.ToInt32(ZeroAngles.Coxa + Direction.X * (workingangles.Coxa * ScalingFactor));
            int FemurServo = Convert.ToInt32(ZeroAngles.Femur + Direction.Y * (workingangles.Femur * ScalingFactor));
            int TibiaServo = Convert.ToInt32(ZeroAngles.Tibia + Direction.Z * (workingangles.Tibia * ScalingFactor));
            LegServoValues ans = new LegServoValues(CoxaServo, FemurServo, TibiaServo); 
            return ans;
        }
    }

    class HeadorTailServoAngles
    {
        public string Type;
        public double Pan;
        public double Tilt;
        public double Aux;

        public HeadorTailServoAngles(double pan, double tilt, double aux, string type)
        {
            Pan = pan;
            Tilt = tilt;
            Aux = aux;
            Type = type;
        }

        public HeadorTailServoValues ConvertToServoValues()
        {
            HeadorTailServoValues ZeroAngles = new HeadorTailServoValues(0,0,0, Type);
            IntVector Direction = new IntVector(0, 0, 0);
            switch (Type)
            {
                case "Head":
                    {
                        ZeroAngles = Hexapod.LegParameters.GetHeadServoValuesAtCentre();
                        Direction = new IntVector(0, 0, 0); //Hexapod.LegParameters.GetServoDirections(LegNumber); //servo reversing
                    }
                    break;
                case "Tail":
                    {
                        ZeroAngles = Hexapod.LegParameters.GetTailServoValuesAtCentre();
                        Direction = new IntVector(0, 0, 0); //Hexapod.LegParameters.GetServoDirections(LegNumber); //servo reversing
                    }
                    break;
            }

            ZeroAngles.Pan *= 4; //from us to 1/4 us
            ZeroAngles.Tilt *= 4;
            ZeroAngles.Aux *= 4;

            double ScalingFactor = (50.49); //Maestro & HXT900
            int PanServo = Convert.ToInt32(ZeroAngles.Pan + Direction.X * (Pan * ScalingFactor));
            int TiltServo = Convert.ToInt32(ZeroAngles.Tilt + Direction.Y * (Tilt * ScalingFactor));
            int AuxServo = Convert.ToInt32(ZeroAngles.Aux + Direction.Z * (Aux * ScalingFactor));
            HeadorTailServoValues ans = new HeadorTailServoValues(PanServo, TiltServo, AuxServo, Type);
            return ans;
        }
    }

    class LegServoValues
    {
        public int Coxa;
        public int Femur;
        public int Tibia;

        public LegServoValues(int coxa, int femur, int tibia)
        {
            Coxa = coxa;
            Femur = femur;
            Tibia = tibia;
        }
    }

    class HeadorTailServoValues
    {
        public string Type;
        public int Pan;
        public int Tilt;
        public int Aux;

        public HeadorTailServoValues(int pan, int tilt, int aux, string type)
        {
            Pan = pan;
            Tilt = tilt;
            Aux = aux;
            Type = type;
        }
    }

}
